{===================================================================================================

  Comming Library.

    TcoMoneyString.

====================================================================================================

  :

  04.02.2003 [*]    CommingDesign  CommingRuntime
             [*]  TAB.
  09.06.1999 [+]   VG Lib.
  09.09.1998 [!]  Comming Library.

====================================================================================================
  Copyright 19982003 TSK Developer Laboratory, Moscow.                     Konstantin Tsvetkov
                                                                              e-mail: dev@tsklab.ru
                                                                              http://www.tsklab.ru
  CVS: $Id: CommingMoneyString.pas,v 1.1 2015/04/04 04:47:46 tsk Exp $
===================================================================================================}
{*******************************************************}
{                                                       }
{         Vladimir Gaitanoff Delphi VCL Library         }
{         TMoneyString                                  }
{         TDateTimeStorage, TCurrencyStorage            }
{                                                       }
{         Copyright (c) 1997, 1998                      }
{                                                       }
{*******************************************************}
//{$D-,L-}

unit CommingMoneyString;

interface

  uses Classes;

  type

{ TcoMoneyString }
  TStringResource = (srMaleOne, srFemaleOne, srTen, srFirstTen, srHundred, srThousand, srMillion, srCurrencySub, srCurrency);

  TMoneyFormat    = (mfVerbal, mfDigit);

  TLanguage        = (lgRussian, lgEnglish, lgSpanish );

  TCurrency        = (crRouble, crDollar );

  TcoMoneyString = class(TComponent)
  private
    FCurrencyGender: array [0..1] of Boolean;
    FDefault: Boolean;
    FDelimeter: string;
    FLists: array [TStringResource] of TStrings;
    FFormats: array [0..1] of TMoneyFormat;
    FLanguage: TLanguage;
    FCurrency: TCurrency;
    FUpperStart: Boolean;
    FValueNumber: Currency;
    FValueString: string;
    FZero: string;
    FZeroEmpty: array [0..1] of Boolean;
    function GetString(Strings: TStrings; Index: Integer): string;
    procedure ListChanged(Sender: TObject);
    function ListStored(Index: Integer): Boolean;
    procedure SetCurrencyGender(Index: Integer; Value: Boolean);
    procedure SetDefault(Value: Boolean);
    procedure SetDelimeter(Value: string);
    procedure SetFormat(Index: Integer; Value: TMoneyFormat);
    procedure SetCurrencyMy(Value: TCurrency);
    procedure SetLanguage(Value: TLanguage);
    procedure SetList(Index: Integer; Value: TStrings);
    procedure SetUpperStart(Value: Boolean);
    procedure SetValueNumber(Value: Currency);
    procedure SetValueString(Value: string);
    procedure SetZero(Value: string);
    procedure SetZeroEmpty(Index: Integer; Value: Boolean);
    function ZeroStored: Boolean;
  public
    constructor Create(AOnwer: TComponent); override;
    destructor Destroy; override;
    function IntegerCurrencyStr(Value: Integer; Currency: TStrings): string;
    function IntegerGenderedToString(Value: Integer; MaleGender: Boolean): string;
    function IntegerToString(Value: Integer): string;
    function MoneyToString(Value: Currency): string;
    function Triada(Value: Integer; MaleGender: Boolean): string;
  published
    property Language: TLanguage read FLanguage write SetLanguage default lgRussian;
    property Currency: TCurrency read FCurrency write SetCurrencyMy default crRouble;
    property NameCurrencySub: TStrings index 7 read FLists[srCurrencySub] write SetList stored ListStored;
    property NameCurrency: TStrings index 8 read FLists[srCurrency] write SetList stored ListStored;
    property Default: Boolean read FDefault write SetDefault stored False default True;
    property Delimeter: string read FDelimeter write SetDelimeter;
    property FemaleOne: TStrings index 1 read FLists[srFemaleOne] write SetList stored ListStored;
    property FirstTen: TStrings index 3 read FLists[srFirstTen] write SetList stored ListStored;
    property FormatCurrency: TMoneyFormat index 0 read FFormats[0] write SetFormat default mfVerbal;
    property FormatCurrencySub: TMoneyFormat index 1 read FFormats[1] write SetFormat default mfVerbal;
    property Hundred: TStrings index 4 read FLists[srHundred] write SetList stored ListStored;
    property MaleCurrency: Boolean index 0 read FCurrencyGender[0] write SetCurrencyGender default True;
    property MaleCurrencySub: Boolean index 1 read FCurrencyGender[1] write SetCurrencyGender;
    property MaleOne: TStrings index 0 read FLists[srMaleOne] write SetList stored ListStored;
    property Thousand: TStrings index 5 read FLists[srThousand] write SetList stored ListStored;
    property Million: TStrings index 6 read FLists[srMillion] write SetList stored ListStored;
    property Ten: TStrings index 2 read FLists[srTen] write SetList stored ListStored;
    property UpperStart: Boolean read FUpperStart write SetUpperStart default True;
    property ValueNumber: Currency read FValueNumber write SetValueNumber stored False;
    property ValueString: string read FValueString write SetValueString stored False;
    property Zero: string read FZero write SetZero stored ZeroStored;
    property ZeroEmptyCurrency: Boolean index 0 read FZeroEmpty[0] write SetZeroEmpty default True;
    property ZeroEmptyCurrencySub: Boolean index 1 read FZeroEmpty[1] write SetZeroEmpty default True;
  end;

//  procedure Register;

implementation

uses SysUtils;

resourcestring
//  ssrRUSZero         = '';
  srrRUSMaleOne     = ''#13''#13''#13''#13''#13''#13''#13''#13'';
  srrRUSFemaleOne   = ''#13''#13''#13''#13''#13''#13''#13''#13'';
  srrRUSTen         =  ''#13''#13''#13''#13''#13''#13''#13''#13'';
  srrRUSFirstTen    = ''#13''#13''#13''#13''#13''#13''#13''#13'';
  srrRUSHundred     = ''#13''#13''#13''#13''#13''#13''#13''#13'';
  srrRUSThousand    = ''#13''#13'';
  srrRUSMillion     = ''#13''#13'';

  srrRUBCurrency    = ''#13''#13'';
  srrRUBCurrencySub = ''#13''#13'';

  srrUSDCurrency    = ''#13''#13'';
  srrUSDCurrencySub = ''#13''#13'';

var StringResourcesDefault: array [TStringResource] of TStrings;
var StringResourcesLoaded: Boolean = False;

//==================================================================================================
{ TcoMoneyString }
//==================================================================================================
constructor TcoMoneyString.Create( AOnwer: TComponent );
  var I: TStringResource;
begin
  inherited;
  if not StringResourcesLoaded then
  begin
    for I := Low(StringResourcesDefault) to High(StringResourcesDefault) do
    begin
      StringResourcesDefault[I] := TStringList.Create;
      case I of
        srMaleOne:      StringResourcesDefault[I].Text := srrRUSMaleOne ;
        srFemaleOne:    StringResourcesDefault[I].Text := srrRUSFemaleOne;
        srTen:          StringResourcesDefault[I].Text := srrRUSTen;
        srFirstTen:     StringResourcesDefault[I].Text := srrRUSFirstTen;
        srHundred:      StringResourcesDefault[I].Text := srrRUSHundred;
        srThousand:     StringResourcesDefault[I].Text := srrRUSThousand;
        srMillion:      StringResourcesDefault[I].Text := srrRUSMillion;

        srCurrency:     StringResourcesDefault[I].Text := srrRUBCurrency;
        srCurrencySub:  StringResourcesDefault[I].Text := srrRUBCurrencySub;
      end;

    end;
    StringResourcesLoaded := True;
  end;

  FCurrencyGender[0] := True;
  FUpperStart := True;
  FZeroEmpty[0] := True;
  FZeroEmpty[1] := True;

  for I := Low(TStringResource) to High(TStringResource) do
  begin
    FLists[I] := TStringList.Create;
    TStringList(FLists[I]).OnChange := ListChanged;
  end;

  SetDefault(True);
end;
//=============================================================================================
destructor TcoMoneyString.Destroy;
var
  I: TStringResource;
begin
  for I := Low(TStringResource) to High(TStringResource) do
    FLists[I].Free;
  inherited;
end;
//=============================================================================================
function TcoMoneyString.GetString(Strings: TStrings; Index: Integer): string;
begin
  if Index < Strings.Count then
    Result := Strings[Index] else
    Result := '';
end;

procedure TcoMoneyString.ListChanged(Sender: TObject);
var
  I: TStringResource;
begin
  for I := Low(TStringResource) to High(TStringResource) do
  begin
    if FLists[I] = Sender then
    begin
      FDefault := not ListStored(Integer(I));
      Break;
    end;
  end;
  SetValueString('');
end;

function TcoMoneyString.ListStored(Index: Integer): Boolean;
begin
  Result := (FLists[TStringResource(Index)].Text <> StringResourcesDefault[TStringResource(Index)].Text);
end;

function TcoMoneyString.Triada(Value:Integer; MaleGender: Boolean): string;
var
  Tmp: Integer;
begin
  Tmp := Value div 100;
  Value := Value mod 100;

  if (Tmp > 0) then
  begin
    Result := GetString(FLists[srHundred], Tmp - 1);
    if Value > 0 then Result := Result + ' ';
  end else
    Result := '';

  Tmp := Value div 10;
  Value := Value mod 10;

  if (Tmp > 1) then
  begin
    Result := Result + GetString(FLists[srTen], Tmp - 1);
    Tmp := Value;
    if (Tmp > 0) then
    begin
      Result := Result + ' ';
      if MaleGender then
        Result := Result + GetString(FLists[srMaleOne], Tmp - 1) else
        Result := Result + GetString(FLists[srFemaleOne], Tmp - 1);
    end;
  end else begin
    Tmp := Tmp * 10 + Value;
    case Tmp of
      1..9:
        begin
          if MaleGender then
            Result := Result + GetString(FLists[srMaleOne], Tmp - 1) else
            Result := Result + GetString(FLists[srFemaleOne], Tmp - 1);
        end;
      10:
        begin
          Result := Result + GetString(FLists[srTen], 0);
        end;
      11..19:
        begin
          Result := Result + GetString(FLists[srFirstTen], Tmp - 11);
        end;
    end;
  end;
end;

function TcoMoneyString.IntegerCurrencyStr(Value: Integer; Currency: TStrings): string;
begin
  if Value mod 100 in [11..19] then
    Result := GetString(Currency, 2)
  else case (Value mod 10) of
    0, 5..9:
      Result := GetString(Currency, 2);
    1:
      Result := GetString(Currency, 0);
    2..4:
      Result := GetString(Currency, 1);
  end;
end;

function TcoMoneyString.IntegerGenderedToString(Value: Integer; MaleGender: Boolean): string;
var
  Tmp: Integer;
begin
  if Value = 0 then
  begin
    Result := FZero;
    Exit;
  end;

  Tmp := Value div 1000000;
  Value := Value mod 1000000;
  if Tmp > 0 then
  begin
    Result := IntegerGenderedToString(Tmp, True) + ' ' +
              IntegerCurrencyStr(Tmp, FLists[srMillion]);
    if Value > 0 then Result := Result + ' ';
  end else
    Result := '';

  Tmp := Value div 1000;
  Value := Value mod 1000;
  if Tmp > 0 then
  begin
    Result := Result +
      IntegerGenderedToString(Tmp, False) + ' ' +
      IntegerCurrencyStr(Tmp, FLists[srThousand]);
    if Value > 0 then Result := Result + ' ';
  end;

  Tmp := Value;
  if (Tmp > 0) then
    Result := Result + Triada(Tmp, MaleGender);
end;

function TcoMoneyString.IntegerToString(Value: Integer): string;
begin
  Result := IntegerGenderedToString(Value, True);
end;

function TcoMoneyString.MoneyToString(Value: Currency): string;
var
  Curr, CurrSub: Integer;
  Tmp, ResultSub: string;
begin
  Curr := Trunc(Value);
  CurrSub := Round(Frac(Value) * 100);

  { Currency string }
  if FFormats[0] = mfVerbal then
  begin
    if not ((Curr = 0) and FZeroEmpty[0]) then
      Result := IntegerGenderedToString(Curr, FCurrencyGender[0]) else
      Result := '';
  end else begin
    if not ((Curr = 0) and FZeroEmpty[0]) then
      Result := Format('%.2d', [Curr]) else
      Result := '';
  end;

  if (Result <> '') then
  begin
    Tmp := IntegerCurrencyStr(Curr, FLists[srCurrency]);
    if Tmp <> '' then Result := Result + ' ' + Tmp;
  end;

  { CurrencySub string }
  if FFormats[1] = mfVerbal then
  begin
    if not ((CurrSub = 0) and FZeroEmpty[1]) then
      ResultSub := IntegerGenderedToString(CurrSub, FCurrencyGender[1]) else
      ResultSub := '';
  end else begin
    if not ((CurrSub = 0) and FZeroEmpty[1]) then
      ResultSub := Format('%.2d', [CurrSub]) else
      ResultSub := '';
  end;

  if (ResultSub <> '') then
  begin
    Tmp := IntegerCurrencyStr(CurrSub, FLists[srCurrencySub]);
    if Tmp <> '' then ResultSub := ResultSub + ' ' + Tmp;
  end;

  { Concate Currency and CurrencySub strings }
  if Result <> '' then
  begin
    if ResultSub <> '' then
    begin
      if FDelimeter = '' then
        Result := Result + ' ' + ResultSub else
        Result := Result + ' ' + Delimeter + ' ' + ResultSub;
    end;
  end else
    Result := Result + ResultSub;

  if FUpperStart and (Length(Result) > 0) then Result[1] := AnsiUpperCase(Result[1])[1];
end;

procedure TcoMoneyString.SetCurrencyGender(Index: Integer; Value: Boolean);
begin
  if FCurrencyGender[Index] <> Value then
  begin
    FCurrencyGender[Index] := Value;
    SetValueString('');
  end;
end;

procedure TcoMoneyString.SetDefault(Value: Boolean);
var
  I: TStringResource;
begin
  if FDefault <> Value then
  begin
    if Value then
    begin
      for I := Low(TStringResource) to High(TStringResource) do
        SetList(Integer(I), StringResourcesDefault[I]);

//      FZero := LoadStr(SsrZero);
    end;
    FDefault := Value;
  end;
end;

procedure TcoMoneyString.SetDelimeter(Value: string);
begin
  if (FDelimeter <> Value) then
  begin
    FDelimeter := Value;
    SetValueString('');
  end;
end;

procedure TcoMoneyString.SetFormat(Index: Integer; Value: TMoneyFormat);
begin
  if (FFormats[Index] <> Value) then
  begin
    FFormats[Index] := Value;
    SetValueString('');
  end;
end;

procedure TcoMoneyString.SetList(Index: Integer; Value: TStrings);
begin
  FLists[TStringResource(Index)].Assign(Value);
end;

procedure TcoMoneyString.SetUpperStart(Value: Boolean);
begin
  if (FUpperStart <> Value) then
  begin
    FUpperStart := Value;
    SetValueString('');
  end;
end;

procedure TcoMoneyString.SetValueNumber(Value: Currency);
begin
  if FValueNumber <> Value then
  begin
    FValueString := MoneyToString(Value);
    FValueNumber := Value;
  end;
end;

procedure TcoMoneyString.SetValueString(Value: string);
begin
  FValueString := MoneyToString(FValueNumber);
end;

procedure TcoMoneyString.SetZero(Value: string);
begin
  if (FZero <> Value) then
  begin
    FZero := Value;
    FDefault := FDefault and not ZeroStored;
    SetValueString('');
  end;
end;

procedure TcoMoneyString.SetZeroEmpty(Index: Integer; Value: Boolean);
begin
  if (FZeroEmpty[Index] <> Value) then
  begin
    FZeroEmpty[Index] := Value;
    SetValueString('');
  end;
end;

function TcoMoneyString.ZeroStored: Boolean;
begin
  Result := True;
//  Result := FZero <> LoadStr(SsrZero);
end;

procedure TcoMoneyString.SetLanguage(Value: TLanguage);
begin

//  if FLanguage = Value then Exit;

{  case FLanguage of
    lgRussian:
      begin
        FLists[srMaleOne].Text := srrRUSMaleOne;
      end;
  end;

 }
end;
//=============================================================================================
procedure TcoMoneyString.SetCurrencyMy( Value: TCurrency );
begin

  if FCurrency = Value then exit;

  case Value of
    crRouble :
      begin
        FLists[ srCurrency ].Text      := srrRUBCurrency;
        FLists[ srCurrencySub ].Text  := srrRUBCurrencySub;
      end;
    crDollar :
      begin
        FLists[ srCurrency ].Text      := srrUSDCurrency;
        FLists[ srCurrencySub ].Text  := srrUSDCurrencySub;
      end;
  end;
end;
//=============================================================================================
{procedure Register;
begin
  RegisterComponents( 'Comming', [TcoMoneyString] );
end;}
//=============================================================================================
end.
